'use strict';

const AWS = require('aws-sdk');

const SNS = new AWS.SNS();

function Emitter (options, query, callback) {
  options = options || {};
  const topic = query.topic || options.topic;
  if (!topic) {
    throw new Error('topic option must be defined');
  }

  const stack = [];
  let running = false;

  function emit (type, data, cb) {
    const payload = {
      id: query.id,
      provider: options.name,
      eventName: query.eventName,
      type: type
    };
    if (data) {
      payload.data = data;
    }
    cb = cb || function () {};

    stack.push([payload, cb]);

    checkStack();
  }

  function checkStack () {
    if (!running && stack.length) {
      const args = stack.shift();
      const payload = args[0];
      const callback = args[1];
      running = true;
      publish(payload, (err) => {
        running = false;
        callback(err);
        setImmediate(() => {
          checkStack();
        });
      });
    }
  }

  function publish (payload, callback) {
    SNS.publish({
      TopicArn: topic,
      Message: JSON.stringify({
        default: JSON.stringify(payload)
      }),
      MessageStructure: 'json'
    }, (err, response) => {
      if (err) {
        options.logger.error(err, `SNS error`);
      } else {
        options.logger.trace(payload, `Emitted event: ${payload.type}`);
      }
      callback(err, response);
    });
  }

  return {
    result: (result) => {
      emit('result', result);
    },
    end: () => {
      emit('end', null, () => { callback(); });
    },
    log: function (level) {
      const args = [].slice.call(arguments, 1);
      try {
        options.logger[level].apply(options.logger, args);
      } catch (e) {}
    }
  };
}

module.exports = Emitter;
