'use strict';

const bunyan = require('bunyan');
const createCloudwatchStream = require('bunyan-cloudwatch');
const defaults = require('lodash.defaults');

function setup (options) {
  options = defaults(options, {
    level: 'info',
    cloudwatch: false
  });

  let streams = [{
    stream: process.stdout,
    level: options.logLevel
  }];

  /* istanbul ignore next */
  if (options.cloudwatch) {
    let stream = createCloudwatchStream({
      logGroupName: options.name,
      logStreamName: options.name,
      cloudWatchLogsOptions: {
        region: 'eu-west-1'
      }
    });
    streams.push({
      level: options.logLevel,
      stream: stream,
      type: 'raw'
    });
  }

  const log = bunyan.createLogger({
    name: options.name,
    level: options.logLevel,
    streams: streams
  });

  return log;
}

module.exports = setup;
